@@ -1,9 +1,9 @@
 """The `version` module holds the version information for Pydantic."""
-from typing import Tuple
+from __future__ import annotations as _annotations
 
 __all__ = 'VERSION', 'version_info'
 
-VERSION = '2.4.1'
+VERSION = '2.6.0'
 """The version of Pydantic."""
 
 
@@ -17,16 +17,15 @@ def version_short() -> str:
 
 def version_info() -> str:
     """Return complete version information for Pydantic and its dependencies."""
+    import importlib.metadata as importlib_metadata
+    import os
     import platform
     import sys
     from pathlib import Path
 
     import pydantic_core._pydantic_core as pdc
 
-    if sys.version_info >= (3, 8):
-        import importlib.metadata as importlib_metadata
-    else:
-        import importlib_metadata
+    from ._internal import _git as git
 
     # get data about packages that are closely related to pydantic, use pydantic or often conflict with pydantic
     package_names = {
@@ -45,6 +44,11 @@ def version_info() -> str:
         if name in package_names:
             related_packages.append(f'{name}-{dist.version}')
 
+    pydantic_dir = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))
+    most_recent_commit = (
+        git.git_revision(pydantic_dir) if git.is_git_repo(pydantic_dir) and git.have_git() else 'unknown'
+    )
+
     info = {
         'pydantic version': VERSION,
         'pydantic-core version': pdc.__version__,
@@ -53,11 +57,12 @@ def version_info() -> str:
         'python version': sys.version,
         'platform': platform.platform(),
         'related packages': ' '.join(related_packages),
+        'commit': most_recent_commit,
     }
     return '\n'.join('{:>30} {}'.format(k + ':', str(v).replace('\n', ' ')) for k, v in info.items())
 
 
-def parse_mypy_version(version: str) -> Tuple[int, ...]:
+def parse_mypy_version(version: str) -> tuple[int, ...]:
     """Parse mypy string version to tuple of ints.
 
     This function is included here rather than the mypy plugin file because the mypy plugin file cannot be imported
